#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include "softcris.h"   /* NOTICE THIS! */
#include "hdfi.h"
#include "crisExpEvt.h"
#include "ce_crisEvt.h"
#include "crisGetData.h"
#include "crisSisGetUtilities.h"
#include "utilities.h"
#include "cris/crisGetData/getEvents_c.h"
#include "mewRecompressedEvent_c.h"

#define VERBOSE 0

#define STRUCT_SIZE 160
#define MAX_DIFF_LINES_TO_WRITE 2000

int main(int argc, char *argv[]) {
    struct mewRecompressedEvent_c *pOldFileStruct, *pNewFileStruct;
    char oldStructBuffer[STRUCT_SIZE], newStructBuffer[STRUCT_SIZE];
    int oldFD, newFD, eventCount, i, j, k, linesWritten;
    char *pOldFilePath, *pNewFilePath;

    if (argc != 3) {
	printf("Usage: compareEventsFiles oldFilePath newFilePath\n");
	return 0;
    }

    pOldFilePath = (char *) argv[1];
    printf("old file = %s\n", pOldFilePath);
    pNewFilePath = (char *) argv[2];
    printf("new file = %s\n", pNewFilePath);

    pOldFileStruct = (struct mewRecompressedEvent_c *) oldStructBuffer;
    pNewFileStruct = (struct mewRecompressedEvent_c *) newStructBuffer;

    oldFD = open(pOldFilePath, O_RDONLY, 0);
    if (oldFD == -1) {
	printf("Error: Can't open %s\n\nExiting.\n\n", pOldFilePath);
	exit(1);
    }
    newFD = open(pNewFilePath, O_RDONLY, 0);
    if (newFD == -1) {
	printf("Error: Can't open %s\n\nExiting.\n\n", pNewFilePath);
	exit(1);
    }

    eventCount = getFilesStructCount(pOldFilePath, STRUCT_SIZE, 0);
    linesWritten = 0;
    for (i = 0; i < eventCount; i++) {
	readFromFile(oldFD, oldStructBuffer, STRUCT_SIZE);
	readFromFile(newFD, newStructBuffer, STRUCT_SIZE);
/*
	if (i == 0) {
	    while (pNewFileStruct->sec96 != pOldFileStruct->sec96)
			readFromFile(newFD, newStructBuffer, STRUCT_SIZE);
	}
*/

	    /* compare sec96 */
	if (VERBOSE || pOldFileStruct->sec96 != pNewFileStruct->sec96) {
	    printf("i = %d, old->sec96 = %ld, new->sec96 = %ld\n",
			i, pOldFileStruct->sec96, pNewFileStruct->sec96);
	    linesWritten++;
	}

	    /* compare sc_clk */
	if (VERBOSE || pOldFileStruct->sc_clk != pNewFileStruct->sc_clk) {
	    printf("i = %d, old->sc_clk = %ld, new->sc_clk = %ld\n", 
			i, pOldFileStruct->sc_clk, pNewFileStruct->sc_clk);
	    linesWritten++;
	}

	    /* compare ev_ofs */
	if (VERBOSE || pOldFileStruct->ev_ofs != pNewFileStruct->ev_ofs) {
	    printf("i = %d, old->ev_ofs = %u, new->ev_ofs = %u\n", 
			i, pOldFileStruct->ev_ofs, pNewFileStruct->ev_ofs);
	    linesWritten++;
	}

	    /* compare ev_len */
	if (VERBOSE || pOldFileStruct->ev_len != pNewFileStruct->ev_len) {
	    printf("i = %d, old->ev_len = %u, new->ev_len = %u\n", 
			i, pOldFileStruct->ev_len, pNewFileStruct->ev_len);
	    linesWritten++;
	}
continue;

	    /* compare flags */
	if (VERBOSE || pOldFileStruct->flags != pNewFileStruct->flags) {
	    printf("i = %d, old->flags = %u, new->flags = %u\n", 
			i, pOldFileStruct->flags, pNewFileStruct->flags);
	    linesWritten++;
	}

	    /* compare bufid */
	if (VERBOSE || pOldFileStruct->bufid != pNewFileStruct->bufid) {
	    printf("i = %d, old->bufid = %u, new->bufid = %u\n", 
			i, pOldFileStruct->bufid, pNewFileStruct->bufid);
	    linesWritten++;
	}

	    /* compare rng_gd */
	if (VERBOSE || pOldFileStruct->rng_gd != pNewFileStruct->rng_gd) {
	    printf("i = %d, old->rng_gd = %u, new->rng_gd = %u\n", 
			i, pOldFileStruct->rng_gd, pNewFileStruct->rng_gd);
	    linesWritten++;
	}

	    /* compare rng_gd2 */
	if (VERBOSE || pOldFileStruct->rng_gd2 != pNewFileStruct->rng_gd2) {
	    printf("i = %d, old->rng_gd2 = %u, new->rng_gd2 = %u\n", 
			i, pOldFileStruct->rng_gd2, pNewFileStruct->rng_gd2);
	    linesWritten++;
	}

	    /* compare stk array */
	for (j = 0; j < 10; j++) {
	    int printMessage;

	    if (VERBOSE) printMessage = 1;
		/* default unused value was changed */
	    else if ((pOldFileStruct->stk[j] == 0) &&
			(pNewFileStruct->stk[j] == 0x0fff)) printMessage = 0;
	    else if (pOldFileStruct->stk[j] != pNewFileStruct->stk[j])
							printMessage = 1;
	    else printMessage = 0;

	    if (printMessage) {
		printf("i = %d, old->stk[%d] = %d, new->stk[%d] = %d\n",
		    i, j, pOldFileStruct->stk[j], j, pNewFileStruct->stk[j]);
		linesWritten++;
	    }
	}

	    /* compare grd array */
	for (j = 0; j < 6; j++) {
	    int printMessage;

	    if (VERBOSE) printMessage = 1;
		/* default unused value was changed */
	    else if ((pOldFileStruct->grd[j] == 0) &&
			(pNewFileStruct->grd[j] == 0x0fff)) printMessage = 0;
	    else if (pOldFileStruct->grd[j] != pNewFileStruct->grd[j])
							printMessage = 1;
	    else printMessage = 0;

	    if (printMessage) {
		printf("i = %d, old->grd[%d] = %d, new->grd[%d] = %d\n",
		    i, j, pOldFileStruct->grd[j], j, pNewFileStruct->grd[j]);
		linesWritten++;
	    }
	}

	    /* compare stim */
	{
	    int printMessage;

	    if (VERBOSE) printMessage = 1;
		/* default unused value was changed */
	    else if ((pOldFileStruct->stim == 0) &&
			(pNewFileStruct->stim == 0x3fff)) printMessage = 0;
	    else if (pOldFileStruct->stim != pNewFileStruct->stim)
							printMessage = 1;
	    else printMessage = 0;

	    if (printMessage) {
		printf("i = %d, old->stim = %d, new->stim = %d\n",
			i, pOldFileStruct->stim, pNewFileStruct->stim);
		linesWritten++;
	    }
	}

	    /* compare npix */
	if (VERBOSE || pOldFileStruct->npix != pNewFileStruct->npix) {
	    printf("i = %d, old->npix = %u, new->npix = %u\n", 
			i, pOldFileStruct->npix, pNewFileStruct->npix);
	    linesWritten++;
	}

	    /* compare soft_par */
	if (VERBOSE || pOldFileStruct->soft_par != pNewFileStruct->soft_par) {
	    printf("i = %d, old->soft_par = %u, new->soft_par = %u\n", 
			i, pOldFileStruct->soft_par, pNewFileStruct->soft_par);
	    linesWritten++;
	}

	    /* compare soft_dat array */
	for (j = 0; j < 18; j++) {
	    for (k = 0; k < 5; k++) {
		int oldVal = pOldFileStruct->soft_dat[j][k];
		int newVal = pNewFileStruct->soft_dat[j][k];

		if (VERBOSE || oldVal != newVal) {
		    printf("i = %d, ", i);
		    printf("old->soft_dat[%d][%d] = %d, ", j, k, oldVal);
		    printf("new->soft_dat[%d][%d] = %d, ", j, k, newVal);
		    printf("diff = %d\n", oldVal - newVal);
		    linesWritten++;
		}
	    }
	}

	    /* compare coord array */
	for (j = 0; j < 3; j++) {
	    for (k = 0; k < 2; k++) {
		int oldVal = pOldFileStruct->coord[j][k];
		int newVal = pNewFileStruct->coord[j][k];

		if (VERBOSE || oldVal != newVal) {
		    printf("i = %d, ", i);
		    printf("old->coord[%d][%d] = %d, ", j, k, oldVal);
		    printf("new->coord[%d][%d] = %d, ", j, k, newVal);
		    printf("diff = %d\n", oldVal - newVal);
		    linesWritten++;
		}
	    }
	}

	    /* compare blob_id */
	if (VERBOSE || pOldFileStruct->blob_id != pNewFileStruct->blob_id) {
	    printf("i = %d, old->blob_id = %ld, new->blob_id = %ld\n",
			i, pOldFileStruct->blob_id, pNewFileStruct->blob_id);
	    linesWritten++;
	}

	    /* compare status array */
	for (j = 0; j < 4; j++) {
	    if (VERBOSE ||
		    pOldFileStruct->status[j] != pNewFileStruct->status[j]) {
		printf("i = %d, old->status[%d] = %d, new->status[%d] = %d\n",
		   i, j, pOldFileStruct->status[j], j, pNewFileStruct->status[j]);
		linesWritten++;
	    }
	}

	if (linesWritten > MAX_DIFF_LINES_TO_WRITE) break;
    }

    return 0;
}
